package org.q.app.config;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.q.toolkit.io.RuntimeIOException;
import org.q.toolkit.lang.Str;

import com.google.inject.Inject;
import com.google.inject.Singleton;

/**
 * This class switches between UserResources and ApplicationResources. If for a relative path, a UserResource is
 * present, the UserResource will be returned. Otherwise, the ApplicationResource will be returned. Ini files are
 * merged, individual entries in the UserResource will override entries in the ApplicationResource.
 * 
 * 
 * @author Erik Lievaart
 */
@Singleton
public class ConfigurableResources implements Resources {

	@Inject
	private UserResources user;
	@Inject
	private ApplicationResources application;

	@Override
	public void storeString(final String path, final String data) throws IOException {
		user.storeString(path, data); // Application configuration files are never written by the application.
	}

	@Override
	public void storeIniFile(final String path, final Map<String, String> properties) throws IOException {
		user.storeIniFile(path, properties); // Application configuration files are never written by the application.
	}

	@Override
	public Map<String, String> loadIniFile(final String path) {
		Map<String, String> properties;
		try {
			properties = application.loadIniFile(path);
		} catch (IOException e) { // original version required
			throw new RuntimeIOException("unable to open resource: " + path, e);
		}
		try {
			properties.putAll(user.loadIniFile(path));
		} catch (IOException e) { // Safely ignore, config file not overridden.
		}
		removeEmptyValues(properties);
		return properties;
	}

	private void removeEmptyValues(final Map<String, String> properties) {
		Iterator<Entry<String, String>> iterator = properties.entrySet().iterator();
		while (iterator.hasNext()) {
			if (Str.isEmpty(iterator.next().getValue())) {
				iterator.remove();
			}
		}
	}

	@Override
	public List<String> listAll(final String path) throws IOException {
		Set<String> set = new HashSet<String>(application.listAll(path));
		set.addAll(user.listAll(path));

		List<String> result = new ArrayList<String>(set);
		Collections.sort(result);
		return result;
	}

	@Override
	public String loadString(final String path) throws IOException {
		try {
			String override = user.loadString(path);
			if (override != null) {
				return override;
			}
		} catch (FileNotFoundException fnfe) {
		}

		return application.loadString(path);
	}

	@Override
	public String getAbsolutePath(final String relative) throws IOException {
		String override = user.getAbsolutePath(relative);
		return new File(override).exists() ? override : application.getAbsolutePath(relative);
	}

}
