package org.q.app.engine.context;

import org.apache.commons.lang3.builder.ToStringBuilder;
import org.q.toolkit.lang.concurrent.Immutable;

/**
 * An immutable snapshot of the left and right file lists. The snapshot stores which locations were open and which files
 * were selected at the time of the snapshot.
 * 
 * @author Erik Lievaart
 */
@Immutable
public class EngineSnapshot {

	private final Active active;
	private final FileSnapshot left;
	private final FileSnapshot right;

	/**
	 * Constructor.
	 * 
	 * @param active
	 *            indicates which file list was active when the snapshot was made.
	 * @param left
	 *            FileSnapshot for the left file list.
	 * @param right
	 *            FileSnapshot for the right file list.
	 */
	public EngineSnapshot(final Active active, final FileSnapshot left, final FileSnapshot right) {
		this.active = active;
		this.left = left;
		this.right = right;
	}

	/**
	 * Get the FileSnapshot for the active file list (at the time of the snapshot).
	 * 
	 * @return the FileSnapshot.
	 */
	public FileSnapshot getActive() {
		return active == Active.LEFT ? left : right;
	}

	/**
	 * Get the FileSnapshot for the inactive file list (at the time of the snapshot).
	 * 
	 * @return the FileSnapshot.
	 */
	public FileSnapshot getInactive() {
		return active == Active.LEFT ? right : left;
	}

	/**
	 * Get the FileSnapshot of the left file list.
	 * 
	 * @return the FileSnapshot.
	 */
	public FileSnapshot getLeft() {
		return left;
	}

	/**
	 * Get the FileSnapshot of the right file list.
	 * 
	 * @return the FileSnapshot.
	 */
	public FileSnapshot getRight() {
		return right;
	}

	@Override
	public String toString() {
		return new ToStringBuilder(this).append("active", getActive().getFile())
				.append("inactive", getInactive().getDir()).toString();
	}
}
