package org.q.app.engine.impl;

import java.lang.reflect.Type;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.builder.ToStringBuilder;
import org.q.app.contract.Invocable;
import org.q.app.contract.PluginContext;
import org.q.app.contract.Swallows;
import org.q.app.contract.ValidationException;
import org.q.app.engine.meta.CommandNameMetadata;
import org.q.app.engine.parse.ShellCommand;
import org.q.toolkit.convert.ConversionException;
import org.q.toolkit.convert.Converters;
import org.q.toolkit.lang.Check;
import org.q.toolkit.lang.FormattedException;
import org.q.toolkit.lang.LogTemplate;
import org.q.toolkit.lang.collection.NewCollection;
import org.q.toolkit.reflect.GenericsTool;
import org.q.toolkit.reflect.InstanceTool;
import org.q.toolkit.reflect.method.MethodWrapper;

import com.google.inject.Inject;
import com.google.inject.Injector;
import com.google.inject.Singleton;

@Singleton
class PluginRunner {

	@Inject
	private Converters convert;
	@Inject
	private Injector injector;
	@Inject
	private PluginMetadata metadata;

	private final LogTemplate log = new LogTemplate(PluginRunner.class);

	void invoke(final ShellCommand command, final PluginContext context) throws Exception {
		Map<String, String[]> flags = metadata.loadFlags(command, context);
		if (command.isInvocable()) {
			invokeInvocable(metadata.getName(command), flags);

		} else {
			invokeSwallowing(metadata.getName(command), flags, command.getSwallowed());
		}
	}

	void invokeSwallowing(final CommandNameMetadata name, final Map<String, String[]> flags, final String raw)
			throws Exception {

		Swallows instance = createObject(name, flags);
		instance.validate(raw);
		instance.invoke(raw);
	}

	void invokeInvocable(final CommandNameMetadata name, final Map<String, String[]> flags) throws Exception {
		Invocable instance = createObject(name, flags);
		instance.validate();
		instance.invoke();
	}

	@SuppressWarnings("unchecked")
	<E> E createObject(final CommandNameMetadata name, final Map<String, String[]> flags) throws ValidationException {

		Object instance = injector.getInstance(name.getLiteral());
		callFlags(instance, flags);

		if (log.isTraceEnabled()) {
			log.trace(ToStringBuilder.reflectionToString(instance));
		}
		return (E) instance;
	}

	void callFlags(final Object instance, final Map<String, String[]> flags) {
		for (String method : flags.keySet()) {
			MethodWrapper wrapper = InstanceTool.getMethodWrapper(method, instance);
			wrapper.invoke(convertFlags(flags.get(method), wrapper.getGenericArgumentTypes()));
		}
	}

	private Object[] convertFlags(final Object[] args, final Type[] types) {
		Check.notNull(args);

		try {
			Object[] out = new Object[args.length];
			for (int i = 0; i < args.length; i++) {
				out[i] = convertValue(types[i], args[i].toString());
			}
			return out;
		} catch (ConversionException e) {
			throw new FormattedException("%; could not convert argument", e, e.getMessage());
		}
	}

	@SuppressWarnings("unchecked")
	Object convertValue(final Type toType, final String value) throws ConversionException {
		if (!GenericsTool.isBaseType(toType, List.class)) {
			return convert.to(GenericsTool.getLiteral(toType), value);
		}
		List<String> strings = convert.to(List.class, value);
		Class<?> genericType = (Class<?>) GenericsTool.getGenericType(toType);

		List converted = NewCollection.list();
		for (String raw : strings) {
			converted.add(convert.to(genericType, raw));
		}
		return converted;
	}

}