package org.q.app.engine.meta;

import java.util.Collections;
import java.util.List;

import org.q.app.contract.ValidationException;
import org.q.app.engine.parse.ShellCommand;
import org.q.app.engine.parse.ShellParseException;
import org.q.app.engine.parse.ShellParser;
import org.q.toolkit.lang.collection.ListTool;
import org.q.toolkit.lang.collection.NewCollection;
import org.q.toolkit.lang.concurrent.Immutable;

import com.google.inject.ProvidedBy;

/**
 * The Meta data of all ShellCommand's (also named plugin's) registered at the application. This is the main entry point
 * of this package for finding information on a plugin. The meta data is read on bootup and is immutable.
 * 
 * @author Erik Lievaart
 */
@Immutable
@ProvidedBy(MetadataCollectionProvider.class)
public class MetadataCollection {
	private final List<CommandMetadata> metadata;

	MetadataCollection(final List<CommandMetadata> metadata) {
		this.metadata = Collections.unmodifiableList(ListTool.sortedCopy(metadata));
	}

	/**
	 * Validate the {@link ShellCommand} against the meta data. If this method completes successfully, then the command
	 * is considered to be semantically correct as far at the meta data can tell. The plugin might still have custom
	 * validation rules.
	 * 
	 * @param parsed
	 *            ShellCommand we want to validate.
	 * @throws ValidationException
	 *             If the command does not validate against the meta data. For instance, the command might not exist.
	 * @throws ShellParseException
	 *             If the command swallows a ShellCommand with syntax errors.
	 */
	public void validate(final ShellCommand parsed) throws ValidationException, ShellParseException {
		CommandMetadata commandMetadata = lookup(parsed.getName());

		MetadataValidator.validate(parsed, commandMetadata);
		if (commandMetadata.isValidateSwallowed() && parsed.hasSwallowed()) {
			validate(ShellParser.parseLine(parsed.getSwallowed()));
		}
	}

	/**
	 * Lookup the meta data for a given {@link ShellCommand}.
	 * 
	 * @param command
	 *            The name of the command we are looking for. This is allowed to be a partial name.
	 * @return The Meta data for the Command if it was found, null otherwise.
	 */
	public CommandMetadata lookup(final String command) {
		return ListTool.lookup(metadata, CommandMetadata.key(command));
	}

	/**
	 * Return a list of all names of plugin's contained in the MetdataCollection.
	 * 
	 * @return the names only.
	 */
	public List<String> listCommands() {
		List<String> commands = NewCollection.list();
		for (CommandMetadata meta : metadata) {
			commands.add(meta.getName().getQName());
		}
		return commands;
	}

}
