package org.q.app.engine.meta;

import java.util.Arrays;
import java.util.Map;

import org.q.app.contract.ValidationException;
import org.q.app.engine.parse.ShellCommand;
import org.q.toolkit.lang.Str;
import org.q.toolkit.lang.collection.NewCollection;
import org.q.toolkit.lang.concurrent.Stateless;

@Stateless
class MetadataValidator {

	private MetadataValidator() {
	}

	static void validate(final ShellCommand command, final CommandMetadata meta) throws ValidationException {
		checkName(command.getName(), meta);
		checkIsRunnable(command.isInvocable(), meta);
		checkSwallows(command.hasSwallowed(), meta);
		checkFlags(command.getFlags(), meta);
	}

	private static void checkName(final String name, final CommandMetadata meta) throws ValidationException {
		boolean check = meta == null || !meta.getName().getQName().startsWith(name);
		ValidationException.on(check, "Unrecognized command: %", name);
	}

	private static void checkIsRunnable(final boolean runnable, final CommandMetadata meta) throws ValidationException {
		boolean isRunnable = meta.getInvocationMetadata().isInvocable();
		ValidationException.on(runnable && !isRunnable, "Swallowed expected for %!", meta.getName().getQName());
	}

	private static void checkSwallows(final boolean swallowed, final CommandMetadata meta) throws ValidationException {
		boolean swallows = meta.getInvocationMetadata().getSwallows();
		ValidationException.on(swallowed && !swallows, "% does not swallow!", meta.getName().getQName());
	}

	private static void checkFlags(final char[] flags, final CommandMetadata meta) throws ValidationException {
		checkFlagsExist(flags, meta);
		checkMutexFlags(flags, meta);
	}

	private static void checkFlagsExist(final char[] flags, final CommandMetadata meta) throws ValidationException {
		char[] metaFlags = meta.getCharacterFlags();

		for (char f : flags) {
			ValidationException.on(Arrays.binarySearch(metaFlags, f) < 0, "Unknown flag %", f);
		}
	}

	private static void checkMutexFlags(final char[] flags, final CommandMetadata meta) throws ValidationException {
		Map<String, Character> groups = NewCollection.map();

		for (char f : flags) {
			String group = meta.getGroup(f);
			boolean duplicate = Str.notBlank(group) && groups.containsKey(group);
			ValidationException.on(duplicate, "Mutually exclusive flags: % %", groups.get(group), f);
			groups.put(group, f);
		}
	}

}
