package org.q.app.engine.script;

import java.io.File;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.vfs.FileObject;
import org.q.app.engine.context.UserVariables;
import org.q.app.vfs.VirtualFile;
import org.q.toolkit.convert.ConversionException;
import org.q.toolkit.convert.Converters;
import org.q.toolkit.lang.Contract;
import org.q.toolkit.lang.collection.NewCollection;

import com.google.inject.Inject;

/**
 * Access the Q variables, these are application settings, the open locations and the selected files.
 * 
 * @author Erik Lievaart
 */
@Contract
public class QVariables {
	private final Map<String, String> variables = NewCollection.map();

	@Inject
	private Converters convert;
	@Inject
	private UserVariables user;

	QVariables() {
	}

	/**
	 * Get the names of all available Q variables.
	 * 
	 * @return the names.
	 */
	public Set<String> getKeys() {
		return variables.keySet();
	}

	/**
	 * Get a Q variable as a String, this returns the raw value.
	 * 
	 * @param key
	 *            name of the variable.
	 * @return the variable.
	 */
	public String getString(final String key) {
		return variables.get(key);
	}

	/**
	 * Get a Q system variable (from variables.ini) as a String, this returns the raw value.
	 * 
	 * @param key
	 *            name of the variable.
	 * @return the variable.
	 */
	public String getSystemVariable(final String key) {
		return user.getVariable(key);
	}

	/**
	 * Get a Q variable as a {@link File}. This only works if the variable is a single File or Folder on the local file
	 * system.
	 * 
	 * @param key
	 *            name of the variable.
	 * @return the file.
	 */
	public File getFile(final String key) {
		return getType(File.class, key);
	}

	/**
	 * Get a Q variable as a Apache VFS {@link FileObject}. This only works if the variable is a single URL.
	 * 
	 * @param key
	 *            name of the variable.
	 * @return the FileObject.
	 */
	public FileObject getFileObject(final String key) {
		return getType(FileObject.class, key);
	}

	/**
	 * Get a Q variable as a {@link VirtualFile}. The {@link VirtualFile} offers access to all possible representations
	 * of the location or part of the url. This only works if the variable is a single URL.
	 * 
	 * @param key
	 *            name of the variable.
	 * @return the VirtualFile.
	 */
	public VirtualFile getVirtualFile(final String key) {
		return getType(VirtualFile.class, key);
	}

	/**
	 * Get the variable as a List of Strings. This is useful for variables that are plural.
	 * 
	 * @param key
	 *            name of the variable.
	 * @return a List of Strings.
	 */
	@SuppressWarnings("unchecked")
	public List<String> getList(final String key) {
		return getType(List.class, key);
	}

	private <T> T getType(final Class<T> type, final String key) {
		if (!variables.containsKey(key)) {
			return null;
		}
		try {
			return convert.to(type, getString(key));
		} catch (ConversionException e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * Not for use in scripts! Put the specified entries in the variable map.
	 * 
	 * @param variables
	 *            Variables to store.
	 */
	void putAll(final Map<String, String> variables) {
		this.variables.putAll(variables);
	}
}
