package org.q.app.engine.script;

import java.util.Hashtable;
import java.util.Map;

import org.q.toolkit.lang.collection.NewCollection;

/**
 * Configuration class for modifying the environment in which a script is executed.
 * 
 * @author Erik Lievaart
 */
public class ScriptConfig {

	private String extension = "bsh";
	private boolean defaultProperties;
	private final Map<String, Object> properties = NewCollection.map();

	/**
	 * Create a ScriptConfig with the default properties that runs a Beanshell script.
	 */
	public ScriptConfig() {
		this(new Hashtable<String, Object>());
	}

	/**
	 * Create a ScriptConfig with the default and some additional properties that runs a Beanshell script.
	 * 
	 * @param properties
	 *            Custom Objects to inject into the script. The keys denote the names of the variables in the script.
	 *            The values denote the actual values injected.
	 */
	public ScriptConfig(final Map<String, ?> properties) {
		this(properties, true);
	}

	/**
	 * Create a ScriptConfig with some additional properties that runs a Beanshell script. The default properties can be
	 * turned on or off.
	 * 
	 * @param properties
	 *            Custom Objects to inject into the script. The keys denote the names of the variables in the script.
	 *            The values denote the actual values injected.
	 * @param defaultProperties
	 *            inject the standard properties into the script. Turning this off is faster but does not allow access
	 *            to the Q environment.
	 */
	public ScriptConfig(final Map<String, ?> properties, final boolean defaultProperties) {
		this.properties.putAll(properties);
		this.defaultProperties = defaultProperties;
	}

	boolean isDefaultProperties() {
		return defaultProperties;
	}

	/**
	 * Turn the default properties on or off.
	 * 
	 * @param defaultProperties
	 *            inject the standard properties into the script. Turning this off is faster but does not allow access
	 *            to the Q environment.
	 */
	public void setDefaultProperties(final boolean defaultProperties) {
		this.defaultProperties = defaultProperties;
	}

	/**
	 * Set the file extension to use when determining which script language to execute. The extension is not used for
	 * finding the file to execute.
	 * 
	 * @param ext
	 *            Extension to use.
	 */
	public void setScriptExtension(final String ext) {
		this.extension = ext;
	}

	String getExtension() {
		return extension;
	}

	/**
	 * Add additional properties.
	 * 
	 * @param map
	 *            Custom Objects to inject into the script. The keys denote the names of the variables in the script.
	 *            The values denote the actual values injected.
	 */
	public void addUserProperties(final Map<String, ?> map) {
		properties.putAll(map);

	}

	Map<String, Object> getUserProperties() {
		return properties;
	}

}
