package org.q.plugins.bookmark;

import org.q.app.config.ResourcePaths;
import org.q.app.contract.Invocable;
import org.q.app.contract.ThreadPolicy;
import org.q.app.contract.ValidationException;
import org.q.app.contract.annotation.Concurrency;
import org.q.app.contract.annotation.Doc;
import org.q.app.contract.annotation.Flag;
import org.q.app.engine.context.BookmarkStore;
import org.q.app.engine.parse.ShellParseException;
import org.q.app.engine.parse.ShellParser;
import org.q.app.engine.script.ScriptInvocation;
import org.q.app.engine.script.SwingScript;
import org.q.toolkit.lang.Check;

import com.google.inject.Inject;

@Concurrency(ThreadPolicy.SWING)
@Doc("Bookmark locations for quick access. Temporary bookmarks will be lost on application restart.")
class BookmarkShellCommand implements Invocable {

	private String name;
	private String value;
	private Mode mode;

	@Inject
	BookmarkStore store;
	@Inject
	private SwingScript script;

	private enum Mode {
		TEMPORARY, PERMANENT, SHOW
	}

	@Flag(group = "main", values = { "``", "$dir" }, primary = true)
	@Doc("In memory bookmark. Two arguments: 1) name of the bookmark (can be empty) 2) file or directory to bookmark")
	public BookmarkShellCommand temporary(final String name, final String bookmark) {
		this.name = name.toLowerCase();
		this.value = bookmark;
		mode = Mode.TEMPORARY;
		return this;
	}

	@Flag(group = "main", values = { "``", "$dir" })
	@Doc("Permanent bookmark. Two arguments: 1) name of the bookmark (can be empty) 2) file or directory to bookmark.")
	public BookmarkShellCommand permanent(final String name, final String bookmark) {
		this.name = name.toLowerCase();
		this.value = bookmark;
		mode = Mode.PERMANENT;
		return this;
	}

	@Flag(group = "main")
	@Doc("Show an overview of all available bookmarks.")
	public BookmarkShellCommand show() {
		mode = Mode.SHOW;
		return this;
	}

	@Override
	public void validate() throws ValidationException {
		if (mode == Mode.TEMPORARY || mode == Mode.PERMANENT) {
			Check.notNull(name, value);
			try {
				ShellParser.parseArgument('@' + name);
			} catch (ShellParseException e) {
				throw new ValidationException("Bookmarks can only contain letters and numbers: " + name);
			}
		}
	}

	@Override
	public void invoke() throws Exception {
		switch (mode) {

			case TEMPORARY:
				store.setVariable(name, value);
				return;

			case PERMANENT:
				store.setPermanentVariable(name, value);
				return;

			case SHOW:
				script.invoke(new ScriptInvocation(ResourcePaths.BOOKMARK_SHOW_SCRIPT));
				return;
		}
		throw new IllegalStateException("Unknown mode: " + mode);
	}
}
