package org.q.plugins.copy;

import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.vfs.FileSystemException;
import org.apache.commons.vfs.Selectors;
import org.q.app.contract.Invocable;
import org.q.app.contract.ThreadPolicy;
import org.q.app.contract.ValidationException;
import org.q.app.contract.annotation.Concurrency;
import org.q.app.contract.annotation.Doc;
import org.q.app.contract.annotation.Flag;
import org.q.app.ui.context.BrowseEvents;
import org.q.app.vfs.VirtualFile;
import org.q.app.vfs.VirtualFiles;
import org.q.toolkit.lang.Check;

import com.google.inject.Inject;

@Concurrency(ThreadPolicy.QUEUE)
@Doc("copy one or more files from any filesystem to any writable filesystem")
class CopyShellCommand implements Invocable {

	private static final String COPY_CHILD_MSG = "% => %; copying to a child of itself!";

	@Inject
	BrowseEvents events;

	private Mode mode;

	private VirtualFile singleFile;
	private String singleRename;
	private VirtualFile destinationDir;
	private List<VirtualFile> urls;

	private enum Mode {
		SINGLE, URLS;
	}

	@Flag(group = "main", values = { "$url", "$dir~", "``" })
	@Doc("Copy a file or directory. Three arguments: 1) file to copy 2) new parent. 3) new file name (if not empty)")
	public CopyShellCommand single(final VirtualFile file, final VirtualFile destination, final String name) {
		singleFile = file;
		destinationDir = destination;
		singleRename = name;
		mode = Mode.SINGLE;

		return this;
	}

	@Flag(group = "main", values = { "$urls", "$dir~" }, primary = true)
	@Doc("Copy multiple urls. Two arguments 1) urls to copy 2) directory to copy urls into")
	public CopyShellCommand urls(final List<VirtualFile> urls, final VirtualFile destination) {
		this.destinationDir = destination;
		this.urls = urls;

		mode = Mode.URLS;
		return this;
	}

	@Override
	public void invoke() throws FileSystemException {
		Check.isTrue(mode != null, "No flags were set!");

		switch (mode) {
		case SINGLE:
			copySingle();
			return;

		case URLS:
			copyUrls();
			return;

		default:
			throw new IllegalStateException("Unknown enum constant: " + mode);
		}
	}

	private void copyUrls() throws FileSystemException {
		for (VirtualFile file : urls) {
			copyFile(file, destinationDir.resolveFile(file.getNameEscaped()));
		}
	}

	private void copySingle() throws FileSystemException {
		String rename = StringUtils.defaultIfEmpty(singleRename, singleFile.getBaseName());
		VirtualFile destinationFile = destinationDir.resolveFile(rename);

		if (VirtualFiles.isSameUrl(singleFile, destinationFile)) {
			destinationFile = destinationDir.resolveFile(singleFile.getBaseName() + "-" + System.currentTimeMillis());
			Check.isFalse(destinationFile.exists(), "% exists!", destinationFile.getName());
		}
		copyFile(singleFile, destinationFile);
	}

	private void copyFile(final VirtualFile source, final VirtualFile destination) throws FileSystemException {
		validateNoCopyToChild(source, destination);
		destination.getFileObject().copyFrom(source.getFileObject(), Selectors.SELECT_ALL);
		events.refresh();
	}

	@Override
	public void validate() throws ValidationException {
		try {
			Check.isTrue(destinationDir.getFileObject().isWriteable(), "Directory not writable: %", destinationDir);
			validateMode();

		} catch (FileSystemException e) {
			throw new ValidationException("Unable to read filesystem; " + e.getMessage(), e);
		}
	}

	private void validateMode() throws FileSystemException, ValidationException {
		switch (mode) {

		case SINGLE:
			validateSingle();
			return;

		case URLS:
			validateUrls();
			return;

		default:
			throw new IllegalStateException("Unknown enum constant: " + mode);
		}
	}

	private void validateSingle() {
		Check.isTrue(singleFile.exists(), "File does not exist: %", singleFile);
		if (!VirtualFiles.isSameUrl(singleFile, destinationDir)) {
			validateNoCopyToChild(singleFile, destinationDir);
		}
	}

	private void validateUrls() throws ValidationException {
		String duplicate = VirtualFiles.findDuplicateFileName(urls);
		ValidationException.unless(duplicate == null, "duplicate name: %", duplicate);
		for (VirtualFile file : urls) {
			validateNoCopyToChild(file, destinationDir);
		}
	}

	private void validateNoCopyToChild(final VirtualFile source, final VirtualFile destination) {
		boolean copyToChild = VirtualFiles.isParentOf(source, destination);
		Check.isFalse(copyToChild, COPY_CHILD_MSG, source.getUrl(), destination.getUrl());
	}

}
