package org.q.plugins.location;

import org.apache.commons.vfs.FileObject;
import org.q.app.contract.Invocable;
import org.q.app.contract.ThreadPolicy;
import org.q.app.contract.ValidationException;
import org.q.app.contract.annotation.Concurrency;
import org.q.app.contract.annotation.Doc;
import org.q.app.contract.annotation.Flag;
import org.q.app.ui.context.BrowseEvents;

import com.google.inject.Inject;

@Concurrency(ThreadPolicy.SWING)
@Doc("Open a location in the filebrowser")
class LocationShellCommand implements Invocable {

	@Inject
	private BrowseEvents events;

	private Event event;
	private String path;
	private FileObject base;
	private String orientation;

	private enum Event {
		URL, HOME, FUZZY, SWAP;
	}

	@Flag(group = "main", values = { "$url", "$dir" })
	@Doc("Open an URL. Two arguments: 1) absolute or relative path 2) base URL (used for relative URL's only).")
	public LocationShellCommand url(final String url, final FileObject relativeTo) {
		event = Event.URL;
		path = url;
		this.base = relativeTo;
		return this;
	}

	@Flag(group = "main", values = { "$url", "$dir" }, primary = true)
	@Doc("Open a fuzzy path. Slower, but more forgiving than the -u flag. Two arguments: 1) path to open 2) base URL")
	public LocationShellCommand fuzzy(final String url, final FileObject relativeTo) {
		event = Event.FUZZY;
		path = url;
		this.base = relativeTo;
		return this;
	}


	@Flag(group = "main")
	@Doc("Swap left and right")
	public LocationShellCommand swap() {
		event = Event.SWAP;
		return this;
	}

	@Flag(group = "orientation", primary = true)
	@Doc("Open location in the active file browser window")
	public LocationShellCommand active() {
		orientation = "active";
		return this;
	}

	@Flag(group = "orientation")
	@Doc("Open location in the inactive file browser window")
	public LocationShellCommand inactive() {
		orientation = "inactive";
		return this;
	}

	@Flag(group = "orientation")
	@Doc("Open location in the left file browser window")
	public LocationShellCommand left() {
		orientation = "left";
		return this;
	}

	@Flag(group = "orientation")
	@Doc("Open location in the right file browser window")
	public LocationShellCommand right() {
		orientation = "right";
		return this;
	}

	@Override
	public void validate() throws ValidationException {
	}

	@Override
	public void invoke() throws Exception {
		switch (event) {

			case SWAP:
				events.swap();
				return;

			case URL:
				events.url(orientation, path, base);
				return;

			case FUZZY:
				events.fuzzy(orientation, path, base);
				return;
		}
		throw new IllegalStateException("Unknown Event " + event);
	}
}
