package org.q.plugins.programming;

import java.util.List;
import java.util.Map;

import javax.script.ScriptEngineFactory;
import javax.script.ScriptEngineManager;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.q.app.contract.Invocable;
import org.q.app.contract.Swallows;
import org.q.app.contract.ThreadPolicy;
import org.q.app.contract.ValidationException;
import org.q.app.contract.annotation.Concurrency;
import org.q.app.contract.annotation.Doc;
import org.q.app.contract.annotation.Flag;
import org.q.app.engine.script.ScriptConfig;
import org.q.app.engine.script.ScriptInvocation;
import org.q.app.engine.script.SwingScript;

import com.google.inject.Inject;

@Concurrency(ThreadPolicy.SWING)
@Doc("Run code in scripting languages")
class ProgrammingShellCommand implements Invocable, Swallows {

	@Inject
	private SwingScript script;

	private final ScriptConfig config = new ScriptConfig();
	private final ScriptInvocation invocation = new ScriptInvocation();
	private boolean dump = false;

	@Flag(group = "main")
	@Doc("Dump a description of all available scripting engines on the command line.")
	public ProgrammingShellCommand dump() {
		dump = true;
		return this;
	}

	@Flag(group = "main", values = "`bsh`", primary = true)
	@Doc("Lookup a scripting engine using its file extension.")
	public ProgrammingShellCommand extension(final String ext) {
		config.setScriptExtension(ext);
		return this;
	}

	@Flag(values = "`default`")
	@Doc("Run all listed resources. These will be resolved to script files.")
	public ProgrammingShellCommand resources(final List<String> files) {
		for (String file : files) {
			if (!StringUtils.isBlank(file)) {
				invocation.addResource(file);
			}
		}
		return this;
	}

	@Flag(values = "`key1=value1#key2=value2`")
	@Doc("Specify custom String variables to set. Variables are separated by '#'. Key / values are separated by '=', ")
	public ProgrammingShellCommand properties(final Map<String, String> variables) {
		config.addUserProperties(variables);
		return this;
	}

	@Override
	public void validate() throws ValidationException {
	}

	@Override
	@Doc("Raw script code to evaluate. In other words: run the swallowed code.")
	public void invoke(final String raw) throws Exception {
		if (dump) {
			dumpLanguages();
			return;
		}
		invocation.setCode(raw);
		script.invoke(config, invocation);
	}

	private void dumpLanguages() {
		Log log = LogFactory.getLog(getClass());
		for (ScriptEngineFactory factory : new ScriptEngineManager().getEngineFactories()) {
			log.info("Language [" + factory.getLanguageName() + "] extensions " + factory.getExtensions());
		}
	}

	@Override
	public void invoke() throws Exception {
		invoke(null);
	}

	@Override
	public void validate(final String raw) throws ValidationException {
	}
}
