package org.q.plugins.various;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.concurrent.atomic.AtomicReference;

import org.apache.commons.lang3.Validate;
import org.q.app.vfs.VirtualFile;
import org.q.app.vfs.VirtualFileComparator;

/**
 * The results of matching files to directories.
 * 
 * @author Erik Lievaart
 */
public class FileMatchResults {
	final static AtomicReference<FileMatchResults> MATCHES = new AtomicReference<FileMatchResults>();

	private final List<Entry<VirtualFile, List<VirtualFile>>> queue;
	private VirtualFile directory;
	private List<VirtualFile> files;
	private final Set<String> moved = new HashSet<String>();

	FileMatchResults(final Map<VirtualFile, List<VirtualFile>> match) {
		Validate.notNull(match);
		queue = new ArrayList<Entry<VirtualFile, List<VirtualFile>>>(match.entrySet());
		sortQueue();
	}

	/**
	 * Get the latest calculated FileMatchResults.
	 */
	// Is there really no better way to do this?
	public static FileMatchResults instance() {
		return MATCHES.get();
	}

	private void sortQueue() {
		Collections.sort(queue, new Comparator<Entry<VirtualFile, List<VirtualFile>>>() {
			@Override
			public int compare(final Entry<VirtualFile, List<VirtualFile>> o1,
					final Entry<VirtualFile, List<VirtualFile>> o2) {
				return new VirtualFileComparator().compare(o1.getKey(), o2.getKey());
			}
		});
	}

	/**
	 * Get the last directory that was matched to one or more files. This is the directory where the files will be moved
	 * if setMoved is called.
	 * 
	 * @see FileMatchResults#setMoved(VirtualFile)
	 */
	public VirtualFile getDirectory() {
		return directory;
	}

	/**
	 * Get the files that were matched to the directory on the last match. These files will can be moved to the
	 * directory if setMoved is called.
	 * 
	 * @see FileMatchResults#setMoved(VirtualFile)
	 */
	public VirtualFile[] getVirtualFiles() {
		return files.toArray(new VirtualFile[] {});
	}

	/**
	 * Move the selected File to the matched directory.
	 * 
	 * @param file
	 *            File to move.
	 */
	public void setMoved(final VirtualFile file) {
		setMoved(new VirtualFile[] { file });
	}

	private void setMoved(final VirtualFile[] all) {
		for (VirtualFile file : all) {
			moved.add(file.getUrlEscaped());
		}
	}

	/**
	 * Go to the next successful match if there is one.
	 */
	public void next() {
		if (queue.isEmpty()) {
			directory = null;
			files = null;
			return;
		}
		removeMovedVirtualFiles();
		directory = queue.get(0).getKey();
		files = queue.get(0).getValue();

		if (files.isEmpty()) {
			next();
		}
		queue.remove(0);
	}

	/**
	 * Returns true only is there are still more successful matches.
	 */
	public boolean hasMoreMatches() {
		removeMovedVirtualFiles();
		return !queue.isEmpty();
	}

	private void removeMovedVirtualFiles() {
		if (queue.isEmpty()) {
			return;
		}
		Iterator<VirtualFile> iter = queue.get(0).getValue().iterator();
		while (iter.hasNext()) {
			if (moved.contains(iter.next().getUrlEscaped())) {
				iter.remove();
			}
		}
		if (queue.get(0).getValue().isEmpty()) {
			queue.remove(0);
			removeMovedVirtualFiles();
		}
	}

}
