package org.q.toolkit.convert;

import org.q.toolkit.convert.validate.AlwaysValidate;
import org.q.toolkit.lang.concurrent.ThreadSafe;

/**
 * Convertors combine validation with construction. Any value successfully validated by the validator should be usable
 * by the Constructor.
 * 
 * @author Erik Lievaart
 * @param <T>
 */
@ThreadSafe
public class Converter<T> {

	private final Constructor<T> constructor;
	private final Validator validator;

	/**
	 * Create a Converter that validates against all input.
	 * 
	 * @param constructor
	 *            create instances using this Constructor.
	 */
	public Converter(final Constructor<T> constructor) {
		this.constructor = constructor;
		validator = new AlwaysValidate();
	}

	/**
	 * Create a Converter that validates against the specified Validator.
	 * 
	 * @param constructor
	 *            create instances using this Constructor.
	 * @param validator
	 *            Validates the input String.
	 */
	public Converter(final Constructor<T> constructor, final Validator validator) {
		this.constructor = constructor;
		this.validator = validator;
	}

	/**
	 * Convert the specified String into an Object.
	 */
	public T convertToObject(final String string) throws ConversionException {
		validator.check(string);
		return constructor.constructObject(string);
	}

	/**
	 * Get the Constructor used by this Converter.
	 */
	public Constructor<T> getConstructor() {
		return constructor;
	}

	/**
	 * Get the Validator used by this Converter.
	 */
	public Validator getValidator() {
		return validator;
	}

	/**
	 * Get the Class literal of instances created by this Converter.
	 */
	public Class<T> getLiteral() {
		return constructor.getLiteral();
	}

}
