package org.q.toolkit.convert;

import java.lang.reflect.Type;
import java.util.Collection;
import java.util.Map;

import org.q.toolkit.lang.Check;
import org.q.toolkit.lang.collection.FromCollection;
import org.q.toolkit.lang.collection.NewCollection;
import org.q.toolkit.lang.concurrent.ThreadSafe;
import org.q.toolkit.reflect.PrimitiveTool;

/**
 * Groups individual Converter's into a collection so that multiple types can be converted by a single Object.
 * 
 * @author Erik Lievaart
 */
@ThreadSafe
public class Converters {

	private final Map<Class<?>, Converter<?>> converters = NewCollection.concurrentMap();

	public Converters(final Converter<?>... converters) {
		this(FromCollection.toList(converters));
	}

	public Converters(final Iterable<Converter<?>> converters) {
		for (Converter<?> converter : converters) {
			Check.notNull(converter);
			Check.notNull(converter.getValidator(), "Validator required!");
			Check.notTrue(PrimitiveTool.isPrimitive(converter.getLiteral()), "Register the wrapper instead");

			this.converters.put(converter.getLiteral(), converter);
		}
	}

	/**
	 * Convert the specified String value to the specified Type.
	 * 
	 * Throws an AssertionException if there is no Converter for the specified Type.
	 */
	@SuppressWarnings("unchecked")
	public <T> T to(final Class<T> type, final String value) throws ConversionException {
		ConversionException.on(!isConvertible(type), "Missing Converter, cannot convert type: %", type);
		return (T) getConverter(type).convertToObject(value);
	}

	/**
	 * Is there a converter for the specified Type?
	 */
	public boolean isConvertible(final Type type) {
		return getConverter(type) != null;
	}

	private Converter<?> getConverter(final Type type) {
		if (PrimitiveTool.isPrimitive(type)) {
			return converters.get(PrimitiveTool.wrapper(type));
		}
		return converters.get(type);
	}

	/**
	 * List all the Converter Objects wrapper by this Converters instance.
	 */
	public Collection<Converter<?>> listConverters() {
		return converters.values();
	}

	/**
	 * List all the Type's that can be converted by this Converters instance.
	 */
	public Collection<Class<?>> getTypes() {
		return converters.keySet();
	}

}
